// This file is part of Hermes3D
//
// Copyright (c) 2009 hp-FEM group at the University of Nevada, Reno (UNR).
// Email: hpfem-group@unr.edu, home page: http://hpfem.org/.
//
// Hermes3D is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published
// by the Free Software Foundation; either version 2 of the License,
// or (at your option) any later version.
//
// Hermes3D is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Hermes3D; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

//
// h1lobbatotetra.cc
//
//

#include "common.h"
#include "h1lobattotetra.h"
#include "lobatto.h"
#include "../../../hermes_common/error.h"
#include "../../../hermes_common/callstack.h"

#ifdef WITH_TETRA

//
// NOTES:
// - Shape functions were generated by gen-shapeset-tetra.pl script (look at utils)
//
//
// Tetrahedral shape functions

// DEGREE 1
//------------

// Vertex shape functions, degree 1

static double lobatto_f0(double x, double y, double z) {
	return lambda1(x, y, z);
}

static double lobatto_f1(double x, double y, double z) {
	return lambda2(x, y, z);
}

static double lobatto_f2(double x, double y, double z) {
	return lambda0(x, y, z);
}

static double lobatto_f3(double x, double y, double z) {
	return lambda3(x, y, z);
}

// DEGREE 2
//------------

// Edge shape functions, degree 2

// edge 0
static double lobatto_f4(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * phi0(lambda1(x, y, z) - lambda2(x, y, z));
}

// edge 1
static double lobatto_f5(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * phi0(lambda2(x, y, z) - lambda0(x, y, z));
}

// edge 2
static double lobatto_f6(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * phi0(lambda1(x, y, z) - lambda0(x, y, z));
}

// edge 3
static double lobatto_f7(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * phi0(lambda1(x, y, z) - lambda3(x, y, z));
}

// edge 4
static double lobatto_f8(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda2(x, y, z) - lambda3(x, y, z));
}

// edge 5
static double lobatto_f9(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda3(x, y, z));
}


// DEGREE 3
//------------

// Edge shape functions, degree 3

// edge 0
static double lobatto_f10_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * phi1(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f10_1(double x, double y, double z) {
	return -lambda1(x, y, z) * lambda2(x, y, z) * phi1(lambda1(x, y, z) - lambda2(x, y, z));
}

// edge 1
static double lobatto_f11_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * phi1(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f11_1(double x, double y, double z) {
	return -lambda2(x, y, z) * lambda0(x, y, z) * phi1(lambda2(x, y, z) - lambda0(x, y, z));
}

// edge 2
static double lobatto_f12_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * phi1(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f12_1(double x, double y, double z) {
	return -lambda1(x, y, z) * lambda0(x, y, z) * phi1(lambda1(x, y, z) - lambda0(x, y, z));
}

// edge 3
static double lobatto_f13_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * phi1(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f13_1(double x, double y, double z) {
	return -lambda1(x, y, z) * lambda3(x, y, z) * phi1(lambda1(x, y, z) - lambda3(x, y, z));
}

// edge 4
static double lobatto_f14_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f14_1(double x, double y, double z) {
	return -lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda2(x, y, z) - lambda3(x, y, z));
}

// edge 5
static double lobatto_f15_0(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f15_1(double x, double y, double z) {
	return -lambda0(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda3(x, y, z));
}


// Face shape functions, degree 3

// face 0
static double lobatto_f16_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f16_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi0(lambda3(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f16_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi0(lambda1(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f16_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi0(lambda3(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f16_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi0(lambda1(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f16_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi0(lambda2(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}


// face 1
static double lobatto_f17_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f17_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi0(lambda3(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f17_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi0(lambda2(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f17_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi0(lambda3(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f17_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda2(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f17_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi0(lambda0(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda2(x, y, z));
}


// face 2
static double lobatto_f18_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f18_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi0(lambda3(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f18_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi0(lambda1(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f18_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi0(lambda3(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f18_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi0(lambda1(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f18_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi0(lambda0(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}


// face 3
static double lobatto_f19_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f19_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi0(lambda0(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f19_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi0(lambda1(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f19_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f19_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi0(lambda1(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f19_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi0(lambda2(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda1(x, y, z));
}


// DEGREE 4
//------------

// Edge shape functions, degree 4

// edge 0
static double lobatto_f20(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * phi2(lambda1(x, y, z) - lambda2(x, y, z));
}

// edge 1
static double lobatto_f21(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * phi2(lambda2(x, y, z) - lambda0(x, y, z));
}

// edge 2
static double lobatto_f22(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * phi2(lambda1(x, y, z) - lambda0(x, y, z));
}

// edge 3
static double lobatto_f23(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * phi2(lambda1(x, y, z) - lambda3(x, y, z));
}

// edge 4
static double lobatto_f24(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda2(x, y, z) - lambda3(x, y, z));
}

// edge 5
static double lobatto_f25(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda3(x, y, z));
}


// Face shape functions, degree 4

// face 0
static double lobatto_f26_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f26_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi0(lambda3(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f26_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi0(lambda1(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f26_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi0(lambda3(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f26_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi0(lambda1(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f26_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi0(lambda2(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f27_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f27_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi1(lambda3(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f27_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi1(lambda1(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f27_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi1(lambda3(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f27_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi1(lambda1(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f27_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi1(lambda2(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}


// face 1
static double lobatto_f28_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f28_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi0(lambda3(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f28_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi0(lambda2(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f28_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi0(lambda3(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f28_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda2(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f28_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi0(lambda0(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f29_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f29_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi1(lambda3(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f29_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi1(lambda2(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f29_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi1(lambda3(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f29_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda2(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f29_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi1(lambda0(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda2(x, y, z));
}


// face 2
static double lobatto_f30_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f30_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi0(lambda3(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f30_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi0(lambda1(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f30_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi0(lambda3(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f30_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi0(lambda1(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f30_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi0(lambda0(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f31_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f31_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi1(lambda3(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f31_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi1(lambda1(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f31_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi1(lambda3(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f31_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi1(lambda1(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f31_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi1(lambda0(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}


// face 3
static double lobatto_f32_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f32_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi0(lambda0(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f32_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi0(lambda1(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f32_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f32_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi0(lambda1(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f32_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi0(lambda2(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f33_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f33_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi1(lambda0(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f33_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi1(lambda1(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f33_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f33_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi1(lambda1(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f33_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi1(lambda2(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda1(x, y, z));
}


// Bubble shape functions, degree 4

static double lobatto_f34(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

// DEGREE 5
//------------

// Edge shape functions, degree 5

// edge 0
static double lobatto_f35_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * phi3(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f35_1(double x, double y, double z) {
	return -lambda1(x, y, z) * lambda2(x, y, z) * phi3(lambda1(x, y, z) - lambda2(x, y, z));
}

// edge 1
static double lobatto_f36_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * phi3(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f36_1(double x, double y, double z) {
	return -lambda2(x, y, z) * lambda0(x, y, z) * phi3(lambda2(x, y, z) - lambda0(x, y, z));
}

// edge 2
static double lobatto_f37_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * phi3(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f37_1(double x, double y, double z) {
	return -lambda1(x, y, z) * lambda0(x, y, z) * phi3(lambda1(x, y, z) - lambda0(x, y, z));
}

// edge 3
static double lobatto_f38_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * phi3(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f38_1(double x, double y, double z) {
	return -lambda1(x, y, z) * lambda3(x, y, z) * phi3(lambda1(x, y, z) - lambda3(x, y, z));
}

// edge 4
static double lobatto_f39_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f39_1(double x, double y, double z) {
	return -lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda2(x, y, z) - lambda3(x, y, z));
}

// edge 5
static double lobatto_f40_0(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f40_1(double x, double y, double z) {
	return -lambda0(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda3(x, y, z));
}


// Face shape functions, degree 5

// face 0
static double lobatto_f41_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f41_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi0(lambda3(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f41_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi0(lambda1(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f41_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi0(lambda3(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f41_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi0(lambda1(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f41_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi0(lambda2(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f42_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f42_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi1(lambda3(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f42_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi1(lambda1(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f42_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi1(lambda3(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f42_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi1(lambda1(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f42_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi1(lambda2(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f43_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f43_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi2(lambda3(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f43_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi2(lambda1(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f43_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi2(lambda3(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f43_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi2(lambda1(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f43_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi2(lambda2(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}


// face 1
static double lobatto_f44_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f44_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi0(lambda3(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f44_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi0(lambda2(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f44_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi0(lambda3(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f44_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda2(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f44_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi0(lambda0(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f45_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f45_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi1(lambda3(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f45_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi1(lambda2(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f45_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi1(lambda3(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f45_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda2(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f45_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi1(lambda0(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f46_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f46_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi2(lambda3(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f46_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi2(lambda2(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f46_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi2(lambda3(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f46_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda2(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f46_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi2(lambda0(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda2(x, y, z));
}


// face 2
static double lobatto_f47_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f47_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi0(lambda3(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f47_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi0(lambda1(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f47_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi0(lambda3(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f47_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi0(lambda1(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f47_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi0(lambda0(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f48_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f48_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi1(lambda3(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f48_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi1(lambda1(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f48_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi1(lambda3(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f48_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi1(lambda1(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f48_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi1(lambda0(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f49_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f49_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi2(lambda3(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f49_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi2(lambda1(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f49_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi2(lambda3(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f49_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi2(lambda1(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f49_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi2(lambda0(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}


// face 3
static double lobatto_f50_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f50_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi0(lambda0(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f50_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi0(lambda1(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f50_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f50_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi0(lambda1(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f50_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi0(lambda2(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f51_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f51_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi1(lambda0(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f51_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi1(lambda1(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f51_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f51_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi1(lambda1(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f51_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi1(lambda2(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f52_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f52_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi2(lambda0(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f52_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi2(lambda1(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f52_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f52_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi2(lambda1(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f52_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi2(lambda2(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda1(x, y, z));
}


// Bubble shape functions, degree 5

static double lobatto_f53(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f54(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f55(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

// DEGREE 6
//------------

// Edge shape functions, degree 6

// edge 0
static double lobatto_f56(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * phi4(lambda1(x, y, z) - lambda2(x, y, z));
}

// edge 1
static double lobatto_f57(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * phi4(lambda2(x, y, z) - lambda0(x, y, z));
}

// edge 2
static double lobatto_f58(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * phi4(lambda1(x, y, z) - lambda0(x, y, z));
}

// edge 3
static double lobatto_f59(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * phi4(lambda1(x, y, z) - lambda3(x, y, z));
}

// edge 4
static double lobatto_f60(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * phi4(lambda2(x, y, z) - lambda3(x, y, z));
}

// edge 5
static double lobatto_f61(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * phi4(lambda0(x, y, z) - lambda3(x, y, z));
}


// Face shape functions, degree 6

// face 0
static double lobatto_f62_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f62_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi0(lambda3(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f62_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi0(lambda1(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f62_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi0(lambda3(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f62_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi0(lambda1(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f62_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi0(lambda2(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f63_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f63_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi1(lambda3(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f63_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi1(lambda1(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f63_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi1(lambda3(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f63_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi1(lambda1(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f63_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi1(lambda2(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f64_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f64_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi2(lambda3(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f64_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi2(lambda1(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f64_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi2(lambda3(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f64_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi2(lambda1(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f64_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi2(lambda2(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f65_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f65_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi3(lambda3(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f65_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi3(lambda1(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f65_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi3(lambda3(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f65_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi3(lambda1(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f65_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi3(lambda2(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}


// face 1
static double lobatto_f66_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f66_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi0(lambda3(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f66_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi0(lambda2(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f66_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi0(lambda3(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f66_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda2(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f66_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi0(lambda0(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f67_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f67_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi1(lambda3(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f67_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi1(lambda2(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f67_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi1(lambda3(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f67_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda2(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f67_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi1(lambda0(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f68_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f68_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi2(lambda3(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f68_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi2(lambda2(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f68_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi2(lambda3(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f68_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda2(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f68_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi2(lambda0(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f69_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f69_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi3(lambda3(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f69_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi3(lambda2(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f69_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi3(lambda3(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f69_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda2(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f69_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi3(lambda0(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda2(x, y, z));
}


// face 2
static double lobatto_f70_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f70_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi0(lambda3(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f70_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi0(lambda1(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f70_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi0(lambda3(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f70_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi0(lambda1(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f70_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi0(lambda0(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f71_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f71_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi1(lambda3(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f71_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi1(lambda1(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f71_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi1(lambda3(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f71_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi1(lambda1(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f71_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi1(lambda0(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f72_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f72_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi2(lambda3(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f72_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi2(lambda1(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f72_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi2(lambda3(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f72_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi2(lambda1(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f72_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi2(lambda0(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f73_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f73_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi3(lambda3(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f73_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi3(lambda1(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f73_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi3(lambda3(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f73_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi3(lambda1(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f73_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi3(lambda0(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}


// face 3
static double lobatto_f74_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f74_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi0(lambda0(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f74_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi0(lambda1(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f74_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f74_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi0(lambda1(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f74_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi0(lambda2(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f75_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f75_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi1(lambda0(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f75_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi1(lambda1(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f75_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f75_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi1(lambda1(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f75_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi1(lambda2(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f76_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f76_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi2(lambda0(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f76_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi2(lambda1(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f76_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f76_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi2(lambda1(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f76_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi2(lambda2(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f77_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f77_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi3(lambda0(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f77_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi3(lambda1(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f77_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f77_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi3(lambda1(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f77_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi3(lambda2(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda1(x, y, z));
}


// Bubble shape functions, degree 6

static double lobatto_f78(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f79(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f80(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f81(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f82(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f83(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

// DEGREE 7
//------------

// Edge shape functions, degree 7

// edge 0
static double lobatto_f84_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * phi5(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f84_1(double x, double y, double z) {
	return -lambda1(x, y, z) * lambda2(x, y, z) * phi5(lambda1(x, y, z) - lambda2(x, y, z));
}

// edge 1
static double lobatto_f85_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * phi5(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f85_1(double x, double y, double z) {
	return -lambda2(x, y, z) * lambda0(x, y, z) * phi5(lambda2(x, y, z) - lambda0(x, y, z));
}

// edge 2
static double lobatto_f86_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * phi5(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f86_1(double x, double y, double z) {
	return -lambda1(x, y, z) * lambda0(x, y, z) * phi5(lambda1(x, y, z) - lambda0(x, y, z));
}

// edge 3
static double lobatto_f87_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * phi5(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f87_1(double x, double y, double z) {
	return -lambda1(x, y, z) * lambda3(x, y, z) * phi5(lambda1(x, y, z) - lambda3(x, y, z));
}

// edge 4
static double lobatto_f88_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * phi5(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f88_1(double x, double y, double z) {
	return -lambda2(x, y, z) * lambda3(x, y, z) * phi5(lambda2(x, y, z) - lambda3(x, y, z));
}

// edge 5
static double lobatto_f89_0(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * phi5(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f89_1(double x, double y, double z) {
	return -lambda0(x, y, z) * lambda3(x, y, z) * phi5(lambda0(x, y, z) - lambda3(x, y, z));
}


// Face shape functions, degree 7

// face 0
static double lobatto_f90_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f90_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi0(lambda3(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f90_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi0(lambda1(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f90_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi0(lambda3(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f90_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi0(lambda1(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f90_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi0(lambda2(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f91_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f91_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi1(lambda3(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f91_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi1(lambda1(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f91_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi1(lambda3(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f91_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi1(lambda1(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f91_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi1(lambda2(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f92_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f92_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi2(lambda3(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f92_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi2(lambda1(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f92_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi2(lambda3(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f92_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi2(lambda1(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f92_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi2(lambda2(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f93_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f93_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi3(lambda3(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f93_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi3(lambda1(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f93_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi3(lambda3(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f93_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi3(lambda1(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f93_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi3(lambda2(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f94_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi4(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f94_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi4(lambda3(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f94_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi4(lambda1(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f94_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi4(lambda3(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f94_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi4(lambda1(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f94_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi4(lambda2(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}


// face 1
static double lobatto_f95_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f95_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi0(lambda3(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f95_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi0(lambda2(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f95_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi0(lambda3(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f95_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda2(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f95_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi0(lambda0(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f96_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f96_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi1(lambda3(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f96_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi1(lambda2(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f96_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi1(lambda3(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f96_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda2(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f96_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi1(lambda0(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f97_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f97_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi2(lambda3(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f97_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi2(lambda2(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f97_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi2(lambda3(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f97_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda2(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f97_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi2(lambda0(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f98_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f98_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi3(lambda3(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f98_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi3(lambda2(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f98_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi3(lambda3(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f98_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda2(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f98_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi3(lambda0(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f99_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi4(lambda0(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f99_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi4(lambda3(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f99_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi4(lambda2(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f99_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi4(lambda3(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f99_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi4(lambda2(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f99_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi4(lambda0(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda2(x, y, z));
}


// face 2
static double lobatto_f100_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f100_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi0(lambda3(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f100_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi0(lambda1(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f100_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi0(lambda3(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f100_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi0(lambda1(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f100_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi0(lambda0(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f101_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f101_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi1(lambda3(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f101_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi1(lambda1(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f101_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi1(lambda3(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f101_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi1(lambda1(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f101_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi1(lambda0(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f102_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f102_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi2(lambda3(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f102_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi2(lambda1(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f102_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi2(lambda3(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f102_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi2(lambda1(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f102_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi2(lambda0(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f103_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f103_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi3(lambda3(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f103_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi3(lambda1(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f103_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi3(lambda3(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f103_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi3(lambda1(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f103_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi3(lambda0(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f104_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi4(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f104_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi4(lambda3(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f104_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi4(lambda1(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f104_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi4(lambda3(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f104_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi4(lambda1(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f104_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi4(lambda0(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}


// face 3
static double lobatto_f105_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f105_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi0(lambda0(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f105_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi0(lambda1(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f105_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f105_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi0(lambda1(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f105_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi0(lambda2(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f106_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f106_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi1(lambda0(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f106_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi1(lambda1(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f106_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f106_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi1(lambda1(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f106_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi1(lambda2(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f107_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f107_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi2(lambda0(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f107_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi2(lambda1(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f107_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f107_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi2(lambda1(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f107_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi2(lambda2(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f108_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f108_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi3(lambda0(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f108_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi3(lambda1(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f108_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f108_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi3(lambda1(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f108_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi3(lambda2(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f109_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi4(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f109_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi4(lambda0(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f109_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi4(lambda1(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f109_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi4(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f109_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi4(lambda1(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f109_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi4(lambda2(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda1(x, y, z));
}


// Bubble shape functions, degree 7

static double lobatto_f110(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f111(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f112(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f113(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f114(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f115(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f116(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f117(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f118(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f119(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

// DEGREE 8
//------------

// Edge shape functions, degree 8

// edge 0
static double lobatto_f120(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * phi6(lambda1(x, y, z) - lambda2(x, y, z));
}

// edge 1
static double lobatto_f121(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * phi6(lambda2(x, y, z) - lambda0(x, y, z));
}

// edge 2
static double lobatto_f122(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * phi6(lambda1(x, y, z) - lambda0(x, y, z));
}

// edge 3
static double lobatto_f123(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * phi6(lambda1(x, y, z) - lambda3(x, y, z));
}

// edge 4
static double lobatto_f124(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * phi6(lambda2(x, y, z) - lambda3(x, y, z));
}

// edge 5
static double lobatto_f125(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * phi6(lambda0(x, y, z) - lambda3(x, y, z));
}


// Face shape functions, degree 8

// face 0
static double lobatto_f126_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi5(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f126_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi0(lambda3(x, y, z) - lambda2(x, y, z)) * phi5(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f126_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi0(lambda1(x, y, z) - lambda3(x, y, z)) * phi5(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f126_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi0(lambda3(x, y, z) - lambda1(x, y, z)) * phi5(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f126_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi0(lambda1(x, y, z) - lambda2(x, y, z)) * phi5(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f126_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi0(lambda2(x, y, z) - lambda3(x, y, z)) * phi5(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f127_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f127_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi1(lambda3(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f127_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi1(lambda1(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f127_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi1(lambda3(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f127_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi1(lambda1(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f127_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi1(lambda2(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f128_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f128_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi2(lambda3(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f128_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi2(lambda1(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f128_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi2(lambda3(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f128_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi2(lambda1(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f128_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi2(lambda2(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f129_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f129_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi3(lambda3(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f129_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi3(lambda1(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f129_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi3(lambda3(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f129_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi3(lambda1(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f129_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi3(lambda2(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f130_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi4(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f130_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi4(lambda3(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f130_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi4(lambda1(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f130_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi4(lambda3(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f130_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi4(lambda1(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f130_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi4(lambda2(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f131_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi5(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f131_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi5(lambda3(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f131_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi5(lambda1(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f131_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi5(lambda3(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f131_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi5(lambda1(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f131_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi5(lambda2(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}


// face 1
static double lobatto_f132_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda2(x, y, z)) * phi5(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f132_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi0(lambda3(x, y, z) - lambda0(x, y, z)) * phi5(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f132_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi0(lambda2(x, y, z) - lambda3(x, y, z)) * phi5(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f132_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi0(lambda3(x, y, z) - lambda2(x, y, z)) * phi5(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f132_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda2(x, y, z) - lambda0(x, y, z)) * phi5(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f132_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi0(lambda0(x, y, z) - lambda3(x, y, z)) * phi5(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f133_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f133_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi1(lambda3(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f133_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi1(lambda2(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f133_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi1(lambda3(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f133_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda2(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f133_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi1(lambda0(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f134_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f134_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi2(lambda3(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f134_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi2(lambda2(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f134_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi2(lambda3(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f134_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda2(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f134_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi2(lambda0(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f135_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f135_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi3(lambda3(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f135_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi3(lambda2(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f135_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi3(lambda3(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f135_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda2(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f135_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi3(lambda0(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f136_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi4(lambda0(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f136_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi4(lambda3(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f136_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi4(lambda2(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f136_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi4(lambda3(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f136_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi4(lambda2(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f136_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi4(lambda0(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f137_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi5(lambda0(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f137_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi5(lambda3(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f137_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi5(lambda2(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f137_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi5(lambda3(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f137_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi5(lambda2(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f137_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi5(lambda0(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda2(x, y, z));
}


// face 2
static double lobatto_f138_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi5(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f138_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi0(lambda3(x, y, z) - lambda0(x, y, z)) * phi5(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f138_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi0(lambda1(x, y, z) - lambda3(x, y, z)) * phi5(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f138_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi0(lambda3(x, y, z) - lambda1(x, y, z)) * phi5(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f138_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi0(lambda1(x, y, z) - lambda0(x, y, z)) * phi5(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f138_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi0(lambda0(x, y, z) - lambda3(x, y, z)) * phi5(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f139_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f139_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi1(lambda3(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f139_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi1(lambda1(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f139_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi1(lambda3(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f139_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi1(lambda1(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f139_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi1(lambda0(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f140_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f140_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi2(lambda3(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f140_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi2(lambda1(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f140_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi2(lambda3(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f140_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi2(lambda1(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f140_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi2(lambda0(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f141_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f141_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi3(lambda3(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f141_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi3(lambda1(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f141_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi3(lambda3(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f141_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi3(lambda1(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f141_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi3(lambda0(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f142_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi4(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f142_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi4(lambda3(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f142_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi4(lambda1(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f142_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi4(lambda3(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f142_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi4(lambda1(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f142_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi4(lambda0(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f143_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi5(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f143_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi5(lambda3(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f143_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi5(lambda1(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f143_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi5(lambda3(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f143_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi5(lambda1(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f143_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi5(lambda0(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}


// face 3
static double lobatto_f144_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi5(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f144_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi0(lambda0(x, y, z) - lambda2(x, y, z)) * phi5(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f144_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi0(lambda1(x, y, z) - lambda0(x, y, z)) * phi5(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f144_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi5(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f144_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi0(lambda1(x, y, z) - lambda2(x, y, z)) * phi5(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f144_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi0(lambda2(x, y, z) - lambda0(x, y, z)) * phi5(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f145_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f145_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi1(lambda0(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f145_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi1(lambda1(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f145_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f145_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi1(lambda1(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f145_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi1(lambda2(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f146_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f146_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi2(lambda0(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f146_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi2(lambda1(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f146_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f146_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi2(lambda1(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f146_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi2(lambda2(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f147_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f147_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi3(lambda0(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f147_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi3(lambda1(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f147_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f147_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi3(lambda1(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f147_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi3(lambda2(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f148_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi4(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f148_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi4(lambda0(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f148_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi4(lambda1(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f148_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi4(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f148_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi4(lambda1(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f148_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi4(lambda2(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f149_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi5(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f149_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi5(lambda0(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f149_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi5(lambda1(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f149_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi5(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f149_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi5(lambda1(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f149_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi5(lambda2(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda1(x, y, z));
}


// Bubble shape functions, degree 8

static double lobatto_f150(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi4(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f151(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f152(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f153(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f154(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi4(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f155(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f156(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f157(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f158(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f159(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f160(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f161(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f162(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f163(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f164(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi4(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

// DEGREE 9
//------------

// Edge shape functions, degree 9

// edge 0
static double lobatto_f165_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * phi7(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f165_1(double x, double y, double z) {
	return -lambda1(x, y, z) * lambda2(x, y, z) * phi7(lambda1(x, y, z) - lambda2(x, y, z));
}

// edge 1
static double lobatto_f166_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * phi7(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f166_1(double x, double y, double z) {
	return -lambda2(x, y, z) * lambda0(x, y, z) * phi7(lambda2(x, y, z) - lambda0(x, y, z));
}

// edge 2
static double lobatto_f167_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * phi7(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f167_1(double x, double y, double z) {
	return -lambda1(x, y, z) * lambda0(x, y, z) * phi7(lambda1(x, y, z) - lambda0(x, y, z));
}

// edge 3
static double lobatto_f168_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * phi7(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f168_1(double x, double y, double z) {
	return -lambda1(x, y, z) * lambda3(x, y, z) * phi7(lambda1(x, y, z) - lambda3(x, y, z));
}

// edge 4
static double lobatto_f169_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * phi7(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f169_1(double x, double y, double z) {
	return -lambda2(x, y, z) * lambda3(x, y, z) * phi7(lambda2(x, y, z) - lambda3(x, y, z));
}

// edge 5
static double lobatto_f170_0(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * phi7(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f170_1(double x, double y, double z) {
	return -lambda0(x, y, z) * lambda3(x, y, z) * phi7(lambda0(x, y, z) - lambda3(x, y, z));
}


// Face shape functions, degree 9

// face 0
static double lobatto_f171_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi6(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f171_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi0(lambda3(x, y, z) - lambda2(x, y, z)) * phi6(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f171_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi0(lambda1(x, y, z) - lambda3(x, y, z)) * phi6(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f171_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi0(lambda3(x, y, z) - lambda1(x, y, z)) * phi6(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f171_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi0(lambda1(x, y, z) - lambda2(x, y, z)) * phi6(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f171_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi0(lambda2(x, y, z) - lambda3(x, y, z)) * phi6(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f172_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi5(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f172_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi1(lambda3(x, y, z) - lambda2(x, y, z)) * phi5(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f172_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi1(lambda1(x, y, z) - lambda3(x, y, z)) * phi5(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f172_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi1(lambda3(x, y, z) - lambda1(x, y, z)) * phi5(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f172_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi1(lambda1(x, y, z) - lambda2(x, y, z)) * phi5(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f172_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi1(lambda2(x, y, z) - lambda3(x, y, z)) * phi5(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f173_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f173_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi2(lambda3(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f173_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi2(lambda1(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f173_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi2(lambda3(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f173_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi2(lambda1(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f173_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi2(lambda2(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f174_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f174_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi3(lambda3(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f174_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi3(lambda1(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f174_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi3(lambda3(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f174_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi3(lambda1(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f174_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi3(lambda2(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f175_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi4(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f175_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi4(lambda3(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f175_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi4(lambda1(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f175_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi4(lambda3(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f175_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi4(lambda1(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f175_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi4(lambda2(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f176_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi5(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f176_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi5(lambda3(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f176_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi5(lambda1(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f176_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi5(lambda3(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f176_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi5(lambda1(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f176_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi5(lambda2(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f177_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi6(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f177_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi6(lambda3(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f177_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi6(lambda1(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f177_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi6(lambda3(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f177_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi6(lambda1(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f177_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi6(lambda2(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}


// face 1
static double lobatto_f178_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda2(x, y, z)) * phi6(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f178_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi0(lambda3(x, y, z) - lambda0(x, y, z)) * phi6(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f178_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi0(lambda2(x, y, z) - lambda3(x, y, z)) * phi6(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f178_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi0(lambda3(x, y, z) - lambda2(x, y, z)) * phi6(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f178_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda2(x, y, z) - lambda0(x, y, z)) * phi6(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f178_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi0(lambda0(x, y, z) - lambda3(x, y, z)) * phi6(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f179_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda2(x, y, z)) * phi5(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f179_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi1(lambda3(x, y, z) - lambda0(x, y, z)) * phi5(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f179_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi1(lambda2(x, y, z) - lambda3(x, y, z)) * phi5(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f179_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi1(lambda3(x, y, z) - lambda2(x, y, z)) * phi5(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f179_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda2(x, y, z) - lambda0(x, y, z)) * phi5(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f179_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi1(lambda0(x, y, z) - lambda3(x, y, z)) * phi5(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f180_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f180_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi2(lambda3(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f180_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi2(lambda2(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f180_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi2(lambda3(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f180_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda2(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f180_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi2(lambda0(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f181_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f181_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi3(lambda3(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f181_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi3(lambda2(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f181_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi3(lambda3(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f181_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda2(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f181_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi3(lambda0(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f182_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi4(lambda0(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f182_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi4(lambda3(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f182_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi4(lambda2(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f182_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi4(lambda3(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f182_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi4(lambda2(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f182_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi4(lambda0(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f183_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi5(lambda0(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f183_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi5(lambda3(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f183_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi5(lambda2(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f183_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi5(lambda3(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f183_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi5(lambda2(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f183_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi5(lambda0(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f184_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi6(lambda0(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f184_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi6(lambda3(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f184_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi6(lambda2(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f184_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi6(lambda3(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f184_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi6(lambda2(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f184_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi6(lambda0(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda2(x, y, z));
}


// face 2
static double lobatto_f185_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi6(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f185_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi0(lambda3(x, y, z) - lambda0(x, y, z)) * phi6(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f185_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi0(lambda1(x, y, z) - lambda3(x, y, z)) * phi6(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f185_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi0(lambda3(x, y, z) - lambda1(x, y, z)) * phi6(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f185_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi0(lambda1(x, y, z) - lambda0(x, y, z)) * phi6(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f185_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi0(lambda0(x, y, z) - lambda3(x, y, z)) * phi6(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f186_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi5(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f186_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi1(lambda3(x, y, z) - lambda0(x, y, z)) * phi5(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f186_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi1(lambda1(x, y, z) - lambda3(x, y, z)) * phi5(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f186_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi1(lambda3(x, y, z) - lambda1(x, y, z)) * phi5(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f186_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi1(lambda1(x, y, z) - lambda0(x, y, z)) * phi5(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f186_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi1(lambda0(x, y, z) - lambda3(x, y, z)) * phi5(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f187_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f187_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi2(lambda3(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f187_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi2(lambda1(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f187_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi2(lambda3(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f187_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi2(lambda1(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f187_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi2(lambda0(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f188_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f188_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi3(lambda3(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f188_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi3(lambda1(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f188_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi3(lambda3(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f188_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi3(lambda1(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f188_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi3(lambda0(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f189_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi4(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f189_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi4(lambda3(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f189_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi4(lambda1(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f189_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi4(lambda3(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f189_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi4(lambda1(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f189_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi4(lambda0(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f190_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi5(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f190_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi5(lambda3(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f190_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi5(lambda1(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f190_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi5(lambda3(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f190_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi5(lambda1(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f190_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi5(lambda0(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f191_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi6(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f191_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi6(lambda3(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f191_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi6(lambda1(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f191_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi6(lambda3(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f191_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi6(lambda1(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f191_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi6(lambda0(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}


// face 3
static double lobatto_f192_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi6(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f192_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi0(lambda0(x, y, z) - lambda2(x, y, z)) * phi6(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f192_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi0(lambda1(x, y, z) - lambda0(x, y, z)) * phi6(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f192_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi6(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f192_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi0(lambda1(x, y, z) - lambda2(x, y, z)) * phi6(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f192_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi0(lambda2(x, y, z) - lambda0(x, y, z)) * phi6(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f193_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi5(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f193_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi1(lambda0(x, y, z) - lambda2(x, y, z)) * phi5(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f193_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi1(lambda1(x, y, z) - lambda0(x, y, z)) * phi5(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f193_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi5(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f193_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi1(lambda1(x, y, z) - lambda2(x, y, z)) * phi5(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f193_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi1(lambda2(x, y, z) - lambda0(x, y, z)) * phi5(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f194_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f194_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi2(lambda0(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f194_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi2(lambda1(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f194_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f194_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi2(lambda1(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f194_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi2(lambda2(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f195_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f195_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi3(lambda0(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f195_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi3(lambda1(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f195_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f195_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi3(lambda1(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f195_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi3(lambda2(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f196_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi4(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f196_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi4(lambda0(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f196_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi4(lambda1(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f196_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi4(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f196_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi4(lambda1(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f196_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi4(lambda2(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f197_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi5(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f197_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi5(lambda0(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f197_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi5(lambda1(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f197_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi5(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f197_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi5(lambda1(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f197_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi5(lambda2(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f198_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi6(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f198_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi6(lambda0(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f198_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi6(lambda1(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f198_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi6(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f198_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi6(lambda1(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f198_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi6(lambda2(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda1(x, y, z));
}


// Bubble shape functions, degree 9

static double lobatto_f199(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi5(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f200(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi4(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f201(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f202(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f203(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi4(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f204(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi5(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f205(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi4(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f206(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f207(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f208(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f209(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi4(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f210(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f211(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f212(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f213(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f214(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f215(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f216(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f217(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi4(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f218(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi4(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f219(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi5(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

// DEGREE 10
//------------

// Edge shape functions, degree 10

// edge 0
static double lobatto_f220(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * phi8(lambda1(x, y, z) - lambda2(x, y, z));
}

// edge 1
static double lobatto_f221(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * phi8(lambda2(x, y, z) - lambda0(x, y, z));
}

// edge 2
static double lobatto_f222(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * phi8(lambda1(x, y, z) - lambda0(x, y, z));
}

// edge 3
static double lobatto_f223(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * phi8(lambda1(x, y, z) - lambda3(x, y, z));
}

// edge 4
static double lobatto_f224(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * phi8(lambda2(x, y, z) - lambda3(x, y, z));
}

// edge 5
static double lobatto_f225(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * phi8(lambda0(x, y, z) - lambda3(x, y, z));
}


// Face shape functions, degree 10

// face 0
static double lobatto_f226_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi7(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f226_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi0(lambda3(x, y, z) - lambda2(x, y, z)) * phi7(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f226_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi0(lambda1(x, y, z) - lambda3(x, y, z)) * phi7(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f226_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi0(lambda3(x, y, z) - lambda1(x, y, z)) * phi7(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f226_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi0(lambda1(x, y, z) - lambda2(x, y, z)) * phi7(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f226_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi0(lambda2(x, y, z) - lambda3(x, y, z)) * phi7(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f227_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi6(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f227_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi1(lambda3(x, y, z) - lambda2(x, y, z)) * phi6(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f227_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi1(lambda1(x, y, z) - lambda3(x, y, z)) * phi6(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f227_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi1(lambda3(x, y, z) - lambda1(x, y, z)) * phi6(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f227_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi1(lambda1(x, y, z) - lambda2(x, y, z)) * phi6(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f227_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi1(lambda2(x, y, z) - lambda3(x, y, z)) * phi6(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f228_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi5(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f228_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi2(lambda3(x, y, z) - lambda2(x, y, z)) * phi5(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f228_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi2(lambda1(x, y, z) - lambda3(x, y, z)) * phi5(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f228_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi2(lambda3(x, y, z) - lambda1(x, y, z)) * phi5(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f228_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi2(lambda1(x, y, z) - lambda2(x, y, z)) * phi5(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f228_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi2(lambda2(x, y, z) - lambda3(x, y, z)) * phi5(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f229_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f229_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi3(lambda3(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f229_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi3(lambda1(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f229_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi3(lambda3(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f229_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi3(lambda1(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f229_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi3(lambda2(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f230_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi4(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f230_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi4(lambda3(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f230_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi4(lambda1(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f230_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi4(lambda3(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f230_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi4(lambda1(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f230_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi4(lambda2(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f231_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi5(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f231_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi5(lambda3(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f231_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi5(lambda1(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f231_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi5(lambda3(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f231_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi5(lambda1(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f231_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi5(lambda2(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f232_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi6(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f232_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi6(lambda3(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f232_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi6(lambda1(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f232_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi6(lambda3(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f232_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi6(lambda1(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f232_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi6(lambda2(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f233_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi7(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f233_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi7(lambda3(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f233_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi7(lambda1(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f233_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi7(lambda3(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f233_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi7(lambda1(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f233_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi7(lambda2(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}


// face 1
static double lobatto_f234_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda2(x, y, z)) * phi7(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f234_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi0(lambda3(x, y, z) - lambda0(x, y, z)) * phi7(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f234_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi0(lambda2(x, y, z) - lambda3(x, y, z)) * phi7(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f234_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi0(lambda3(x, y, z) - lambda2(x, y, z)) * phi7(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f234_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda2(x, y, z) - lambda0(x, y, z)) * phi7(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f234_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi0(lambda0(x, y, z) - lambda3(x, y, z)) * phi7(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f235_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda2(x, y, z)) * phi6(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f235_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi1(lambda3(x, y, z) - lambda0(x, y, z)) * phi6(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f235_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi1(lambda2(x, y, z) - lambda3(x, y, z)) * phi6(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f235_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi1(lambda3(x, y, z) - lambda2(x, y, z)) * phi6(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f235_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda2(x, y, z) - lambda0(x, y, z)) * phi6(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f235_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi1(lambda0(x, y, z) - lambda3(x, y, z)) * phi6(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f236_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda2(x, y, z)) * phi5(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f236_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi2(lambda3(x, y, z) - lambda0(x, y, z)) * phi5(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f236_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi2(lambda2(x, y, z) - lambda3(x, y, z)) * phi5(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f236_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi2(lambda3(x, y, z) - lambda2(x, y, z)) * phi5(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f236_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda2(x, y, z) - lambda0(x, y, z)) * phi5(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f236_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi2(lambda0(x, y, z) - lambda3(x, y, z)) * phi5(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f237_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f237_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi3(lambda3(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f237_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi3(lambda2(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f237_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi3(lambda3(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f237_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda2(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f237_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi3(lambda0(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f238_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi4(lambda0(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f238_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi4(lambda3(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f238_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi4(lambda2(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f238_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi4(lambda3(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f238_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi4(lambda2(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f238_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi4(lambda0(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f239_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi5(lambda0(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f239_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi5(lambda3(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f239_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi5(lambda2(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f239_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi5(lambda3(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f239_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi5(lambda2(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f239_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi5(lambda0(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f240_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi6(lambda0(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f240_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi6(lambda3(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f240_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi6(lambda2(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f240_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi6(lambda3(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f240_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi6(lambda2(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f240_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi6(lambda0(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f241_0(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi7(lambda0(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f241_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda2(x, y, z) * phi7(lambda3(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f241_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi7(lambda2(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f241_3(double x, double y, double z) {
	return lambda2(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi7(lambda3(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f241_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi7(lambda2(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f241_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi7(lambda0(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda2(x, y, z));
}


// face 2
static double lobatto_f242_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi7(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f242_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi0(lambda3(x, y, z) - lambda0(x, y, z)) * phi7(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f242_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi0(lambda1(x, y, z) - lambda3(x, y, z)) * phi7(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f242_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi0(lambda3(x, y, z) - lambda1(x, y, z)) * phi7(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f242_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi0(lambda1(x, y, z) - lambda0(x, y, z)) * phi7(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f242_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi0(lambda0(x, y, z) - lambda3(x, y, z)) * phi7(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f243_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi6(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f243_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi1(lambda3(x, y, z) - lambda0(x, y, z)) * phi6(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f243_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi1(lambda1(x, y, z) - lambda3(x, y, z)) * phi6(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f243_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi1(lambda3(x, y, z) - lambda1(x, y, z)) * phi6(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f243_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi1(lambda1(x, y, z) - lambda0(x, y, z)) * phi6(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f243_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi1(lambda0(x, y, z) - lambda3(x, y, z)) * phi6(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f244_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi5(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f244_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi2(lambda3(x, y, z) - lambda0(x, y, z)) * phi5(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f244_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi2(lambda1(x, y, z) - lambda3(x, y, z)) * phi5(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f244_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi2(lambda3(x, y, z) - lambda1(x, y, z)) * phi5(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f244_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi2(lambda1(x, y, z) - lambda0(x, y, z)) * phi5(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f244_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi2(lambda0(x, y, z) - lambda3(x, y, z)) * phi5(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f245_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f245_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi3(lambda3(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f245_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi3(lambda1(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f245_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi3(lambda3(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f245_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi3(lambda1(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f245_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi3(lambda0(x, y, z) - lambda3(x, y, z)) * phi4(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f246_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi4(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f246_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi4(lambda3(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f246_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi4(lambda1(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f246_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi4(lambda3(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f246_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi4(lambda1(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f246_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi4(lambda0(x, y, z) - lambda3(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f247_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi5(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f247_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi5(lambda3(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f247_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi5(lambda1(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f247_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi5(lambda3(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f247_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi5(lambda1(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f247_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi5(lambda0(x, y, z) - lambda3(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f248_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi6(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f248_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi6(lambda3(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f248_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi6(lambda1(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f248_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi6(lambda3(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f248_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi6(lambda1(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f248_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi6(lambda0(x, y, z) - lambda3(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f249_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda3(x, y, z) * phi7(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f249_1(double x, double y, double z) {
	return lambda0(x, y, z) * lambda3(x, y, z) * lambda1(x, y, z) * phi7(lambda3(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f249_2(double x, double y, double z) {
	return lambda3(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi7(lambda1(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f249_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda3(x, y, z) * lambda0(x, y, z) * phi7(lambda3(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f249_4(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda3(x, y, z) * phi7(lambda1(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda3(x, y, z));
}

static double lobatto_f249_5(double x, double y, double z) {
	return lambda3(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi7(lambda0(x, y, z) - lambda3(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}


// face 3
static double lobatto_f250_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi7(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f250_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi0(lambda0(x, y, z) - lambda2(x, y, z)) * phi7(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f250_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi0(lambda1(x, y, z) - lambda0(x, y, z)) * phi7(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f250_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi7(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f250_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi0(lambda1(x, y, z) - lambda2(x, y, z)) * phi7(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f250_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi0(lambda2(x, y, z) - lambda0(x, y, z)) * phi7(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f251_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi6(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f251_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi1(lambda0(x, y, z) - lambda2(x, y, z)) * phi6(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f251_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi1(lambda1(x, y, z) - lambda0(x, y, z)) * phi6(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f251_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi6(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f251_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi1(lambda1(x, y, z) - lambda2(x, y, z)) * phi6(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f251_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi1(lambda2(x, y, z) - lambda0(x, y, z)) * phi6(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f252_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi5(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f252_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi2(lambda0(x, y, z) - lambda2(x, y, z)) * phi5(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f252_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi2(lambda1(x, y, z) - lambda0(x, y, z)) * phi5(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f252_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi5(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f252_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi2(lambda1(x, y, z) - lambda2(x, y, z)) * phi5(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f252_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi2(lambda2(x, y, z) - lambda0(x, y, z)) * phi5(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f253_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f253_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi3(lambda0(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f253_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi3(lambda1(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f253_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi4(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f253_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi3(lambda1(x, y, z) - lambda2(x, y, z)) * phi4(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f253_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi3(lambda2(x, y, z) - lambda0(x, y, z)) * phi4(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f254_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi4(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f254_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi4(lambda0(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f254_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi4(lambda1(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f254_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi4(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f254_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi4(lambda1(x, y, z) - lambda2(x, y, z)) * phi3(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f254_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi4(lambda2(x, y, z) - lambda0(x, y, z)) * phi3(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f255_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi5(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f255_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi5(lambda0(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f255_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi5(lambda1(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f255_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi5(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f255_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi5(lambda1(x, y, z) - lambda2(x, y, z)) * phi2(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f255_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi5(lambda2(x, y, z) - lambda0(x, y, z)) * phi2(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f256_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi6(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f256_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi6(lambda0(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f256_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi6(lambda1(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f256_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi6(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f256_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi6(lambda1(x, y, z) - lambda2(x, y, z)) * phi1(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f256_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi6(lambda2(x, y, z) - lambda0(x, y, z)) * phi1(lambda0(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f257_0(double x, double y, double z) {
	return lambda1(x, y, z) * lambda2(x, y, z) * lambda0(x, y, z) * phi7(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f257_1(double x, double y, double z) {
	return lambda2(x, y, z) * lambda0(x, y, z) * lambda1(x, y, z) * phi7(lambda0(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f257_2(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * phi7(lambda1(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f257_3(double x, double y, double z) {
	return lambda1(x, y, z) * lambda0(x, y, z) * lambda2(x, y, z) * phi7(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda1(x, y, z) - lambda2(x, y, z));
}

static double lobatto_f257_4(double x, double y, double z) {
	return lambda2(x, y, z) * lambda1(x, y, z) * lambda0(x, y, z) * phi7(lambda1(x, y, z) - lambda2(x, y, z)) * phi0(lambda2(x, y, z) - lambda0(x, y, z));
}

static double lobatto_f257_5(double x, double y, double z) {
	return lambda0(x, y, z) * lambda2(x, y, z) * lambda1(x, y, z) * phi7(lambda2(x, y, z) - lambda0(x, y, z)) * phi0(lambda0(x, y, z) - lambda1(x, y, z));
}


// Bubble shape functions, degree 10

static double lobatto_f258(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi6(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f259(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi5(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f260(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi4(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f261(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f262(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi4(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f263(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi5(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f264(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi0(lambda0(x, y, z) - lambda1(x, y, z)) * phi6(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f265(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi5(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f266(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi4(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f267(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f268(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f269(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi4(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f270(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi1(lambda0(x, y, z) - lambda1(x, y, z)) * phi5(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f271(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi4(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f272(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f273(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f274(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f275(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi2(lambda0(x, y, z) - lambda1(x, y, z)) * phi4(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f276(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi3(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f277(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f278(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f279(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi3(lambda0(x, y, z) - lambda1(x, y, z)) * phi3(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f280(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi4(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi2(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f281(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi4(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f282(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi4(lambda0(x, y, z) - lambda1(x, y, z)) * phi2(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f283(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi5(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi1(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f284(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi5(lambda0(x, y, z) - lambda1(x, y, z)) * phi1(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}

static double lobatto_f285(double x, double y, double z) {
	return lambda0(x, y, z) * lambda1(x, y, z) * lambda2(x, y, z) * lambda3(x, y, z) * phi6(lambda0(x, y, z) - lambda1(x, y, z)) * phi0(lambda2(x, y, z) - lambda1(x, y, z)) * phi0(lambda3(x, y, z) - lambda1(x, y, z));
}


shape_fn_t lobatto_tetra_fn[] = {
	lobatto_f0, lobatto_f1, lobatto_f2, lobatto_f3, lobatto_f4, lobatto_f5, lobatto_f6, lobatto_f7,
	lobatto_f8, lobatto_f9, lobatto_f10_0, lobatto_f10_1, lobatto_f11_0, lobatto_f11_1, lobatto_f12_0, lobatto_f12_1,
	lobatto_f13_0, lobatto_f13_1, lobatto_f14_0, lobatto_f14_1, lobatto_f15_0, lobatto_f15_1, lobatto_f16_0, lobatto_f16_1,
	lobatto_f16_2, lobatto_f16_3, lobatto_f16_4, lobatto_f16_5, lobatto_f17_0, lobatto_f17_1, lobatto_f17_2, lobatto_f17_3,
	lobatto_f17_4, lobatto_f17_5, lobatto_f18_0, lobatto_f18_1, lobatto_f18_2, lobatto_f18_3, lobatto_f18_4, lobatto_f18_5,
	lobatto_f19_0, lobatto_f19_1, lobatto_f19_2, lobatto_f19_3, lobatto_f19_4, lobatto_f19_5, lobatto_f20, lobatto_f21,
	lobatto_f22, lobatto_f23, lobatto_f24, lobatto_f25, lobatto_f26_0, lobatto_f26_1, lobatto_f26_2, lobatto_f26_3,
	lobatto_f26_4, lobatto_f26_5, lobatto_f27_0, lobatto_f27_1, lobatto_f27_2, lobatto_f27_3, lobatto_f27_4, lobatto_f27_5,
	lobatto_f28_0, lobatto_f28_1, lobatto_f28_2, lobatto_f28_3, lobatto_f28_4, lobatto_f28_5, lobatto_f29_0, lobatto_f29_1,
	lobatto_f29_2, lobatto_f29_3, lobatto_f29_4, lobatto_f29_5, lobatto_f30_0, lobatto_f30_1, lobatto_f30_2, lobatto_f30_3,
	lobatto_f30_4, lobatto_f30_5, lobatto_f31_0, lobatto_f31_1, lobatto_f31_2, lobatto_f31_3, lobatto_f31_4, lobatto_f31_5,
	lobatto_f32_0, lobatto_f32_1, lobatto_f32_2, lobatto_f32_3, lobatto_f32_4, lobatto_f32_5, lobatto_f33_0, lobatto_f33_1,
	lobatto_f33_2, lobatto_f33_3, lobatto_f33_4, lobatto_f33_5, lobatto_f34, lobatto_f35_0, lobatto_f35_1, lobatto_f36_0,
	lobatto_f36_1, lobatto_f37_0, lobatto_f37_1, lobatto_f38_0, lobatto_f38_1, lobatto_f39_0, lobatto_f39_1, lobatto_f40_0,
	lobatto_f40_1, lobatto_f41_0, lobatto_f41_1, lobatto_f41_2, lobatto_f41_3, lobatto_f41_4, lobatto_f41_5, lobatto_f42_0,
	lobatto_f42_1, lobatto_f42_2, lobatto_f42_3, lobatto_f42_4, lobatto_f42_5, lobatto_f43_0, lobatto_f43_1, lobatto_f43_2,
	lobatto_f43_3, lobatto_f43_4, lobatto_f43_5, lobatto_f44_0, lobatto_f44_1, lobatto_f44_2, lobatto_f44_3, lobatto_f44_4,
	lobatto_f44_5, lobatto_f45_0, lobatto_f45_1, lobatto_f45_2, lobatto_f45_3, lobatto_f45_4, lobatto_f45_5, lobatto_f46_0,
	lobatto_f46_1, lobatto_f46_2, lobatto_f46_3, lobatto_f46_4, lobatto_f46_5, lobatto_f47_0, lobatto_f47_1, lobatto_f47_2,
	lobatto_f47_3, lobatto_f47_4, lobatto_f47_5, lobatto_f48_0, lobatto_f48_1, lobatto_f48_2, lobatto_f48_3, lobatto_f48_4,
	lobatto_f48_5, lobatto_f49_0, lobatto_f49_1, lobatto_f49_2, lobatto_f49_3, lobatto_f49_4, lobatto_f49_5, lobatto_f50_0,
	lobatto_f50_1, lobatto_f50_2, lobatto_f50_3, lobatto_f50_4, lobatto_f50_5, lobatto_f51_0, lobatto_f51_1, lobatto_f51_2,
	lobatto_f51_3, lobatto_f51_4, lobatto_f51_5, lobatto_f52_0, lobatto_f52_1, lobatto_f52_2, lobatto_f52_3, lobatto_f52_4,
	lobatto_f52_5, lobatto_f53, lobatto_f54, lobatto_f55, lobatto_f56, lobatto_f57, lobatto_f58, lobatto_f59,
	lobatto_f60, lobatto_f61, lobatto_f62_0, lobatto_f62_1, lobatto_f62_2, lobatto_f62_3, lobatto_f62_4, lobatto_f62_5,
	lobatto_f63_0, lobatto_f63_1, lobatto_f63_2, lobatto_f63_3, lobatto_f63_4, lobatto_f63_5, lobatto_f64_0, lobatto_f64_1,
	lobatto_f64_2, lobatto_f64_3, lobatto_f64_4, lobatto_f64_5, lobatto_f65_0, lobatto_f65_1, lobatto_f65_2, lobatto_f65_3,
	lobatto_f65_4, lobatto_f65_5, lobatto_f66_0, lobatto_f66_1, lobatto_f66_2, lobatto_f66_3, lobatto_f66_4, lobatto_f66_5,
	lobatto_f67_0, lobatto_f67_1, lobatto_f67_2, lobatto_f67_3, lobatto_f67_4, lobatto_f67_5, lobatto_f68_0, lobatto_f68_1,
	lobatto_f68_2, lobatto_f68_3, lobatto_f68_4, lobatto_f68_5, lobatto_f69_0, lobatto_f69_1, lobatto_f69_2, lobatto_f69_3,
	lobatto_f69_4, lobatto_f69_5, lobatto_f70_0, lobatto_f70_1, lobatto_f70_2, lobatto_f70_3, lobatto_f70_4, lobatto_f70_5,
	lobatto_f71_0, lobatto_f71_1, lobatto_f71_2, lobatto_f71_3, lobatto_f71_4, lobatto_f71_5, lobatto_f72_0, lobatto_f72_1,
	lobatto_f72_2, lobatto_f72_3, lobatto_f72_4, lobatto_f72_5, lobatto_f73_0, lobatto_f73_1, lobatto_f73_2, lobatto_f73_3,
	lobatto_f73_4, lobatto_f73_5, lobatto_f74_0, lobatto_f74_1, lobatto_f74_2, lobatto_f74_3, lobatto_f74_4, lobatto_f74_5,
	lobatto_f75_0, lobatto_f75_1, lobatto_f75_2, lobatto_f75_3, lobatto_f75_4, lobatto_f75_5, lobatto_f76_0, lobatto_f76_1,
	lobatto_f76_2, lobatto_f76_3, lobatto_f76_4, lobatto_f76_5, lobatto_f77_0, lobatto_f77_1, lobatto_f77_2, lobatto_f77_3,
	lobatto_f77_4, lobatto_f77_5, lobatto_f78, lobatto_f79, lobatto_f80, lobatto_f81, lobatto_f82, lobatto_f83,
	lobatto_f84_0, lobatto_f84_1, lobatto_f85_0, lobatto_f85_1, lobatto_f86_0, lobatto_f86_1, lobatto_f87_0, lobatto_f87_1,
	lobatto_f88_0, lobatto_f88_1, lobatto_f89_0, lobatto_f89_1, lobatto_f90_0, lobatto_f90_1, lobatto_f90_2, lobatto_f90_3,
	lobatto_f90_4, lobatto_f90_5, lobatto_f91_0, lobatto_f91_1, lobatto_f91_2, lobatto_f91_3, lobatto_f91_4, lobatto_f91_5,
	lobatto_f92_0, lobatto_f92_1, lobatto_f92_2, lobatto_f92_3, lobatto_f92_4, lobatto_f92_5, lobatto_f93_0, lobatto_f93_1,
	lobatto_f93_2, lobatto_f93_3, lobatto_f93_4, lobatto_f93_5, lobatto_f94_0, lobatto_f94_1, lobatto_f94_2, lobatto_f94_3,
	lobatto_f94_4, lobatto_f94_5, lobatto_f95_0, lobatto_f95_1, lobatto_f95_2, lobatto_f95_3, lobatto_f95_4, lobatto_f95_5,
	lobatto_f96_0, lobatto_f96_1, lobatto_f96_2, lobatto_f96_3, lobatto_f96_4, lobatto_f96_5, lobatto_f97_0, lobatto_f97_1,
	lobatto_f97_2, lobatto_f97_3, lobatto_f97_4, lobatto_f97_5, lobatto_f98_0, lobatto_f98_1, lobatto_f98_2, lobatto_f98_3,
	lobatto_f98_4, lobatto_f98_5, lobatto_f99_0, lobatto_f99_1, lobatto_f99_2, lobatto_f99_3, lobatto_f99_4, lobatto_f99_5,
	lobatto_f100_0, lobatto_f100_1, lobatto_f100_2, lobatto_f100_3, lobatto_f100_4, lobatto_f100_5, lobatto_f101_0, lobatto_f101_1,
	lobatto_f101_2, lobatto_f101_3, lobatto_f101_4, lobatto_f101_5, lobatto_f102_0, lobatto_f102_1, lobatto_f102_2, lobatto_f102_3,
	lobatto_f102_4, lobatto_f102_5, lobatto_f103_0, lobatto_f103_1, lobatto_f103_2, lobatto_f103_3, lobatto_f103_4, lobatto_f103_5,
	lobatto_f104_0, lobatto_f104_1, lobatto_f104_2, lobatto_f104_3, lobatto_f104_4, lobatto_f104_5, lobatto_f105_0, lobatto_f105_1,
	lobatto_f105_2, lobatto_f105_3, lobatto_f105_4, lobatto_f105_5, lobatto_f106_0, lobatto_f106_1, lobatto_f106_2, lobatto_f106_3,
	lobatto_f106_4, lobatto_f106_5, lobatto_f107_0, lobatto_f107_1, lobatto_f107_2, lobatto_f107_3, lobatto_f107_4, lobatto_f107_5,
	lobatto_f108_0, lobatto_f108_1, lobatto_f108_2, lobatto_f108_3, lobatto_f108_4, lobatto_f108_5, lobatto_f109_0, lobatto_f109_1,
	lobatto_f109_2, lobatto_f109_3, lobatto_f109_4, lobatto_f109_5, lobatto_f110, lobatto_f111, lobatto_f112, lobatto_f113,
	lobatto_f114, lobatto_f115, lobatto_f116, lobatto_f117, lobatto_f118, lobatto_f119, lobatto_f120, lobatto_f121,
	lobatto_f122, lobatto_f123, lobatto_f124, lobatto_f125, lobatto_f126_0, lobatto_f126_1, lobatto_f126_2, lobatto_f126_3,
	lobatto_f126_4, lobatto_f126_5, lobatto_f127_0, lobatto_f127_1, lobatto_f127_2, lobatto_f127_3, lobatto_f127_4, lobatto_f127_5,
	lobatto_f128_0, lobatto_f128_1, lobatto_f128_2, lobatto_f128_3, lobatto_f128_4, lobatto_f128_5, lobatto_f129_0, lobatto_f129_1,
	lobatto_f129_2, lobatto_f129_3, lobatto_f129_4, lobatto_f129_5, lobatto_f130_0, lobatto_f130_1, lobatto_f130_2, lobatto_f130_3,
	lobatto_f130_4, lobatto_f130_5, lobatto_f131_0, lobatto_f131_1, lobatto_f131_2, lobatto_f131_3, lobatto_f131_4, lobatto_f131_5,
	lobatto_f132_0, lobatto_f132_1, lobatto_f132_2, lobatto_f132_3, lobatto_f132_4, lobatto_f132_5, lobatto_f133_0, lobatto_f133_1,
	lobatto_f133_2, lobatto_f133_3, lobatto_f133_4, lobatto_f133_5, lobatto_f134_0, lobatto_f134_1, lobatto_f134_2, lobatto_f134_3,
	lobatto_f134_4, lobatto_f134_5, lobatto_f135_0, lobatto_f135_1, lobatto_f135_2, lobatto_f135_3, lobatto_f135_4, lobatto_f135_5,
	lobatto_f136_0, lobatto_f136_1, lobatto_f136_2, lobatto_f136_3, lobatto_f136_4, lobatto_f136_5, lobatto_f137_0, lobatto_f137_1,
	lobatto_f137_2, lobatto_f137_3, lobatto_f137_4, lobatto_f137_5, lobatto_f138_0, lobatto_f138_1, lobatto_f138_2, lobatto_f138_3,
	lobatto_f138_4, lobatto_f138_5, lobatto_f139_0, lobatto_f139_1, lobatto_f139_2, lobatto_f139_3, lobatto_f139_4, lobatto_f139_5,
	lobatto_f140_0, lobatto_f140_1, lobatto_f140_2, lobatto_f140_3, lobatto_f140_4, lobatto_f140_5, lobatto_f141_0, lobatto_f141_1,
	lobatto_f141_2, lobatto_f141_3, lobatto_f141_4, lobatto_f141_5, lobatto_f142_0, lobatto_f142_1, lobatto_f142_2, lobatto_f142_3,
	lobatto_f142_4, lobatto_f142_5, lobatto_f143_0, lobatto_f143_1, lobatto_f143_2, lobatto_f143_3, lobatto_f143_4, lobatto_f143_5,
	lobatto_f144_0, lobatto_f144_1, lobatto_f144_2, lobatto_f144_3, lobatto_f144_4, lobatto_f144_5, lobatto_f145_0, lobatto_f145_1,
	lobatto_f145_2, lobatto_f145_3, lobatto_f145_4, lobatto_f145_5, lobatto_f146_0, lobatto_f146_1, lobatto_f146_2, lobatto_f146_3,
	lobatto_f146_4, lobatto_f146_5, lobatto_f147_0, lobatto_f147_1, lobatto_f147_2, lobatto_f147_3, lobatto_f147_4, lobatto_f147_5,
	lobatto_f148_0, lobatto_f148_1, lobatto_f148_2, lobatto_f148_3, lobatto_f148_4, lobatto_f148_5, lobatto_f149_0, lobatto_f149_1,
	lobatto_f149_2, lobatto_f149_3, lobatto_f149_4, lobatto_f149_5, lobatto_f150, lobatto_f151, lobatto_f152, lobatto_f153,
	lobatto_f154, lobatto_f155, lobatto_f156, lobatto_f157, lobatto_f158, lobatto_f159, lobatto_f160, lobatto_f161,
	lobatto_f162, lobatto_f163, lobatto_f164, lobatto_f165_0, lobatto_f165_1, lobatto_f166_0, lobatto_f166_1, lobatto_f167_0,
	lobatto_f167_1, lobatto_f168_0, lobatto_f168_1, lobatto_f169_0, lobatto_f169_1, lobatto_f170_0, lobatto_f170_1, lobatto_f171_0,
	lobatto_f171_1, lobatto_f171_2, lobatto_f171_3, lobatto_f171_4, lobatto_f171_5, lobatto_f172_0, lobatto_f172_1, lobatto_f172_2,
	lobatto_f172_3, lobatto_f172_4, lobatto_f172_5, lobatto_f173_0, lobatto_f173_1, lobatto_f173_2, lobatto_f173_3, lobatto_f173_4,
	lobatto_f173_5, lobatto_f174_0, lobatto_f174_1, lobatto_f174_2, lobatto_f174_3, lobatto_f174_4, lobatto_f174_5, lobatto_f175_0,
	lobatto_f175_1, lobatto_f175_2, lobatto_f175_3, lobatto_f175_4, lobatto_f175_5, lobatto_f176_0, lobatto_f176_1, lobatto_f176_2,
	lobatto_f176_3, lobatto_f176_4, lobatto_f176_5, lobatto_f177_0, lobatto_f177_1, lobatto_f177_2, lobatto_f177_3, lobatto_f177_4,
	lobatto_f177_5, lobatto_f178_0, lobatto_f178_1, lobatto_f178_2, lobatto_f178_3, lobatto_f178_4, lobatto_f178_5, lobatto_f179_0,
	lobatto_f179_1, lobatto_f179_2, lobatto_f179_3, lobatto_f179_4, lobatto_f179_5, lobatto_f180_0, lobatto_f180_1, lobatto_f180_2,
	lobatto_f180_3, lobatto_f180_4, lobatto_f180_5, lobatto_f181_0, lobatto_f181_1, lobatto_f181_2, lobatto_f181_3, lobatto_f181_4,
	lobatto_f181_5, lobatto_f182_0, lobatto_f182_1, lobatto_f182_2, lobatto_f182_3, lobatto_f182_4, lobatto_f182_5, lobatto_f183_0,
	lobatto_f183_1, lobatto_f183_2, lobatto_f183_3, lobatto_f183_4, lobatto_f183_5, lobatto_f184_0, lobatto_f184_1, lobatto_f184_2,
	lobatto_f184_3, lobatto_f184_4, lobatto_f184_5, lobatto_f185_0, lobatto_f185_1, lobatto_f185_2, lobatto_f185_3, lobatto_f185_4,
	lobatto_f185_5, lobatto_f186_0, lobatto_f186_1, lobatto_f186_2, lobatto_f186_3, lobatto_f186_4, lobatto_f186_5, lobatto_f187_0,
	lobatto_f187_1, lobatto_f187_2, lobatto_f187_3, lobatto_f187_4, lobatto_f187_5, lobatto_f188_0, lobatto_f188_1, lobatto_f188_2,
	lobatto_f188_3, lobatto_f188_4, lobatto_f188_5, lobatto_f189_0, lobatto_f189_1, lobatto_f189_2, lobatto_f189_3, lobatto_f189_4,
	lobatto_f189_5, lobatto_f190_0, lobatto_f190_1, lobatto_f190_2, lobatto_f190_3, lobatto_f190_4, lobatto_f190_5, lobatto_f191_0,
	lobatto_f191_1, lobatto_f191_2, lobatto_f191_3, lobatto_f191_4, lobatto_f191_5, lobatto_f192_0, lobatto_f192_1, lobatto_f192_2,
	lobatto_f192_3, lobatto_f192_4, lobatto_f192_5, lobatto_f193_0, lobatto_f193_1, lobatto_f193_2, lobatto_f193_3, lobatto_f193_4,
	lobatto_f193_5, lobatto_f194_0, lobatto_f194_1, lobatto_f194_2, lobatto_f194_3, lobatto_f194_4, lobatto_f194_5, lobatto_f195_0,
	lobatto_f195_1, lobatto_f195_2, lobatto_f195_3, lobatto_f195_4, lobatto_f195_5, lobatto_f196_0, lobatto_f196_1, lobatto_f196_2,
	lobatto_f196_3, lobatto_f196_4, lobatto_f196_5, lobatto_f197_0, lobatto_f197_1, lobatto_f197_2, lobatto_f197_3, lobatto_f197_4,
	lobatto_f197_5, lobatto_f198_0, lobatto_f198_1, lobatto_f198_2, lobatto_f198_3, lobatto_f198_4, lobatto_f198_5, lobatto_f199,
	lobatto_f200, lobatto_f201, lobatto_f202, lobatto_f203, lobatto_f204, lobatto_f205, lobatto_f206, lobatto_f207,
	lobatto_f208, lobatto_f209, lobatto_f210, lobatto_f211, lobatto_f212, lobatto_f213, lobatto_f214, lobatto_f215,
	lobatto_f216, lobatto_f217, lobatto_f218, lobatto_f219, lobatto_f220, lobatto_f221, lobatto_f222, lobatto_f223,
	lobatto_f224, lobatto_f225, lobatto_f226_0, lobatto_f226_1, lobatto_f226_2, lobatto_f226_3, lobatto_f226_4, lobatto_f226_5,
	lobatto_f227_0, lobatto_f227_1, lobatto_f227_2, lobatto_f227_3, lobatto_f227_4, lobatto_f227_5, lobatto_f228_0, lobatto_f228_1,
	lobatto_f228_2, lobatto_f228_3, lobatto_f228_4, lobatto_f228_5, lobatto_f229_0, lobatto_f229_1, lobatto_f229_2, lobatto_f229_3,
	lobatto_f229_4, lobatto_f229_5, lobatto_f230_0, lobatto_f230_1, lobatto_f230_2, lobatto_f230_3, lobatto_f230_4, lobatto_f230_5,
	lobatto_f231_0, lobatto_f231_1, lobatto_f231_2, lobatto_f231_3, lobatto_f231_4, lobatto_f231_5, lobatto_f232_0, lobatto_f232_1,
	lobatto_f232_2, lobatto_f232_3, lobatto_f232_4, lobatto_f232_5, lobatto_f233_0, lobatto_f233_1, lobatto_f233_2, lobatto_f233_3,
	lobatto_f233_4, lobatto_f233_5, lobatto_f234_0, lobatto_f234_1, lobatto_f234_2, lobatto_f234_3, lobatto_f234_4, lobatto_f234_5,
	lobatto_f235_0, lobatto_f235_1, lobatto_f235_2, lobatto_f235_3, lobatto_f235_4, lobatto_f235_5, lobatto_f236_0, lobatto_f236_1,
	lobatto_f236_2, lobatto_f236_3, lobatto_f236_4, lobatto_f236_5, lobatto_f237_0, lobatto_f237_1, lobatto_f237_2, lobatto_f237_3,
	lobatto_f237_4, lobatto_f237_5, lobatto_f238_0, lobatto_f238_1, lobatto_f238_2, lobatto_f238_3, lobatto_f238_4, lobatto_f238_5,
	lobatto_f239_0, lobatto_f239_1, lobatto_f239_2, lobatto_f239_3, lobatto_f239_4, lobatto_f239_5, lobatto_f240_0, lobatto_f240_1,
	lobatto_f240_2, lobatto_f240_3, lobatto_f240_4, lobatto_f240_5, lobatto_f241_0, lobatto_f241_1, lobatto_f241_2, lobatto_f241_3,
	lobatto_f241_4, lobatto_f241_5, lobatto_f242_0, lobatto_f242_1, lobatto_f242_2, lobatto_f242_3, lobatto_f242_4, lobatto_f242_5,
	lobatto_f243_0, lobatto_f243_1, lobatto_f243_2, lobatto_f243_3, lobatto_f243_4, lobatto_f243_5, lobatto_f244_0, lobatto_f244_1,
	lobatto_f244_2, lobatto_f244_3, lobatto_f244_4, lobatto_f244_5, lobatto_f245_0, lobatto_f245_1, lobatto_f245_2, lobatto_f245_3,
	lobatto_f245_4, lobatto_f245_5, lobatto_f246_0, lobatto_f246_1, lobatto_f246_2, lobatto_f246_3, lobatto_f246_4, lobatto_f246_5,
	lobatto_f247_0, lobatto_f247_1, lobatto_f247_2, lobatto_f247_3, lobatto_f247_4, lobatto_f247_5, lobatto_f248_0, lobatto_f248_1,
	lobatto_f248_2, lobatto_f248_3, lobatto_f248_4, lobatto_f248_5, lobatto_f249_0, lobatto_f249_1, lobatto_f249_2, lobatto_f249_3,
	lobatto_f249_4, lobatto_f249_5, lobatto_f250_0, lobatto_f250_1, lobatto_f250_2, lobatto_f250_3, lobatto_f250_4, lobatto_f250_5,
	lobatto_f251_0, lobatto_f251_1, lobatto_f251_2, lobatto_f251_3, lobatto_f251_4, lobatto_f251_5, lobatto_f252_0, lobatto_f252_1,
	lobatto_f252_2, lobatto_f252_3, lobatto_f252_4, lobatto_f252_5, lobatto_f253_0, lobatto_f253_1, lobatto_f253_2, lobatto_f253_3,
	lobatto_f253_4, lobatto_f253_5, lobatto_f254_0, lobatto_f254_1, lobatto_f254_2, lobatto_f254_3, lobatto_f254_4, lobatto_f254_5,
	lobatto_f255_0, lobatto_f255_1, lobatto_f255_2, lobatto_f255_3, lobatto_f255_4, lobatto_f255_5, lobatto_f256_0, lobatto_f256_1,
	lobatto_f256_2, lobatto_f256_3, lobatto_f256_4, lobatto_f256_5, lobatto_f257_0, lobatto_f257_1, lobatto_f257_2, lobatto_f257_3,
	lobatto_f257_4, lobatto_f257_5, lobatto_f258, lobatto_f259, lobatto_f260, lobatto_f261, lobatto_f262, lobatto_f263,
	lobatto_f264, lobatto_f265, lobatto_f266, lobatto_f267, lobatto_f268, lobatto_f269, lobatto_f270, lobatto_f271,
	lobatto_f272, lobatto_f273, lobatto_f274, lobatto_f275, lobatto_f276, lobatto_f277, lobatto_f278, lobatto_f279,
	lobatto_f280, lobatto_f281, lobatto_f282, lobatto_f283, lobatto_f284, lobatto_f285
};

// vertices //

static int lobatto_tetra_vertex_indices[] = { 0, 1, 2, 3 };

// edges //

static int lobatto_tetra_edge_indices_0_0[] = { 4, 10, 46, 101, 188, 296, 438, 603, 804 };
static int lobatto_tetra_edge_indices_0_1[] = { 4, 11, 46, 102, 188, 297, 438, 604, 804 };

static int *lobatto_tetra_edge_indices_0[] = {
	lobatto_tetra_edge_indices_0_0,
	lobatto_tetra_edge_indices_0_1
};

static int lobatto_tetra_edge_indices_1_0[] = { 5, 12, 47, 103, 189, 298, 439, 605, 805 };
static int lobatto_tetra_edge_indices_1_1[] = { 5, 13, 47, 104, 189, 299, 439, 606, 805 };

static int *lobatto_tetra_edge_indices_1[] = {
	lobatto_tetra_edge_indices_1_0,
	lobatto_tetra_edge_indices_1_1
};

static int lobatto_tetra_edge_indices_2_0[] = { 6, 14, 48, 105, 190, 300, 440, 607, 806 };
static int lobatto_tetra_edge_indices_2_1[] = { 6, 15, 48, 106, 190, 301, 440, 608, 806 };

static int *lobatto_tetra_edge_indices_2[] = {
	lobatto_tetra_edge_indices_2_0,
	lobatto_tetra_edge_indices_2_1
};

static int lobatto_tetra_edge_indices_3_0[] = { 7, 16, 49, 107, 191, 302, 441, 609, 807 };
static int lobatto_tetra_edge_indices_3_1[] = { 7, 17, 49, 108, 191, 303, 441, 610, 807 };

static int *lobatto_tetra_edge_indices_3[] = {
	lobatto_tetra_edge_indices_3_0,
	lobatto_tetra_edge_indices_3_1
};

static int lobatto_tetra_edge_indices_4_0[] = { 8, 18, 50, 109, 192, 304, 442, 611, 808 };
static int lobatto_tetra_edge_indices_4_1[] = { 8, 19, 50, 110, 192, 305, 442, 612, 808 };

static int *lobatto_tetra_edge_indices_4[] = {
	lobatto_tetra_edge_indices_4_0,
	lobatto_tetra_edge_indices_4_1
};

static int lobatto_tetra_edge_indices_5_0[] = { 9, 20, 51, 111, 193, 306, 443, 613, 809 };
static int lobatto_tetra_edge_indices_5_1[] = { 9, 21, 51, 112, 193, 307, 443, 614, 809 };

static int *lobatto_tetra_edge_indices_5[] = {
	lobatto_tetra_edge_indices_5_0,
	lobatto_tetra_edge_indices_5_1
};

static int **lobatto_tetra_edge_indices[] = {
	lobatto_tetra_edge_indices_0,
	lobatto_tetra_edge_indices_1,
	lobatto_tetra_edge_indices_2,
	lobatto_tetra_edge_indices_3,
	lobatto_tetra_edge_indices_4,
	lobatto_tetra_edge_indices_5
};

static int lobatto_tetra_edge_count[] = { 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1 };

// faces //

static int lobatto_tetra_face_indices_0_0[] = {
	22, 52, 58, 113, 119, 125, 194, 200, 206, 212, 308, 314, 320, 326, 332, 444, 450, 456, 462, 468, 474, 615, 621, 627, 633, 639, 645, 651, 810, 816, 822, 828, 834, 840, 846, 852
};

static int lobatto_tetra_face_indices_0_1[] = {
	23, 53, 59, 114, 120, 126, 195, 201, 207, 213, 309, 315, 321, 327, 333, 445, 451, 457, 463, 469, 475, 616, 622, 628, 634, 640, 646, 652, 811, 817, 823, 829, 835, 841, 847, 853
};

static int lobatto_tetra_face_indices_0_2[] = {
	24, 54, 60, 115, 121, 127, 196, 202, 208, 214, 310, 316, 322, 328, 334, 446, 452, 458, 464, 470, 476, 617, 623, 629, 635, 641, 647, 653, 812, 818, 824, 830, 836, 842, 848, 854
};

static int lobatto_tetra_face_indices_0_3[] = {
	25, 55, 61, 116, 122, 128, 197, 203, 209, 215, 311, 317, 323, 329, 335, 447, 453, 459, 465, 471, 477, 618, 624, 630, 636, 642, 648, 654, 813, 819, 825, 831, 837, 843, 849, 855
};

static int lobatto_tetra_face_indices_0_4[] = {
	26, 56, 62, 117, 123, 129, 198, 204, 210, 216, 312, 318, 324, 330, 336, 448, 454, 460, 466, 472, 478, 619, 625, 631, 637, 643, 649, 655, 814, 820, 826, 832, 838, 844, 850, 856
};

static int lobatto_tetra_face_indices_0_5[] = {
	27, 57, 63, 118, 124, 130, 199, 205, 211, 217, 313, 319, 325, 331, 337, 449, 455, 461, 467, 473, 479, 620, 626, 632, 638, 644, 650, 656, 815, 821, 827, 833, 839, 845, 851, 857
};

static int *lobatto_tetra_face_indices_0[] = {
	lobatto_tetra_face_indices_0_0,
	lobatto_tetra_face_indices_0_1,
	lobatto_tetra_face_indices_0_2,
	lobatto_tetra_face_indices_0_3,
	lobatto_tetra_face_indices_0_4,
	lobatto_tetra_face_indices_0_5
};

static int lobatto_tetra_face_indices_1_0[] = {
	28, 64, 70, 131, 137, 143, 218, 224, 230, 236, 338, 344, 350, 356, 362, 480, 486, 492, 498, 504, 510, 657, 663, 669, 675, 681, 687, 693, 858, 864, 870, 876, 882, 888, 894, 900
};

static int lobatto_tetra_face_indices_1_1[] = {
	29, 65, 71, 132, 138, 144, 219, 225, 231, 237, 339, 345, 351, 357, 363, 481, 487, 493, 499, 505, 511, 658, 664, 670, 676, 682, 688, 694, 859, 865, 871, 877, 883, 889, 895, 901
};

static int lobatto_tetra_face_indices_1_2[] = {
	30, 66, 72, 133, 139, 145, 220, 226, 232, 238, 340, 346, 352, 358, 364, 482, 488, 494, 500, 506, 512, 659, 665, 671, 677, 683, 689, 695, 860, 866, 872, 878, 884, 890, 896, 902
};

static int lobatto_tetra_face_indices_1_3[] = {
	31, 67, 73, 134, 140, 146, 221, 227, 233, 239, 341, 347, 353, 359, 365, 483, 489, 495, 501, 507, 513, 660, 666, 672, 678, 684, 690, 696, 861, 867, 873, 879, 885, 891, 897, 903
};

static int lobatto_tetra_face_indices_1_4[] = {
	32, 68, 74, 135, 141, 147, 222, 228, 234, 240, 342, 348, 354, 360, 366, 484, 490, 496, 502, 508, 514, 661, 667, 673, 679, 685, 691, 697, 862, 868, 874, 880, 886, 892, 898, 904
};

static int lobatto_tetra_face_indices_1_5[] = {
	33, 69, 75, 136, 142, 148, 223, 229, 235, 241, 343, 349, 355, 361, 367, 485, 491, 497, 503, 509, 515, 662, 668, 674, 680, 686, 692, 698, 863, 869, 875, 881, 887, 893, 899, 905
};

static int *lobatto_tetra_face_indices_1[] = {
	lobatto_tetra_face_indices_1_0,
	lobatto_tetra_face_indices_1_1,
	lobatto_tetra_face_indices_1_2,
	lobatto_tetra_face_indices_1_3,
	lobatto_tetra_face_indices_1_4,
	lobatto_tetra_face_indices_1_5
};

static int lobatto_tetra_face_indices_2_0[] = {
	34, 76, 82, 149, 155, 161, 242, 248, 254, 260, 368, 374, 380, 386, 392, 516, 522, 528, 534, 540, 546, 699, 705, 711, 717, 723, 729, 735, 906, 912, 918, 924, 930, 936, 942, 948
};

static int lobatto_tetra_face_indices_2_1[] = {
	35, 77, 83, 150, 156, 162, 243, 249, 255, 261, 369, 375, 381, 387, 393, 517, 523, 529, 535, 541, 547, 700, 706, 712, 718, 724, 730, 736, 907, 913, 919, 925, 931, 937, 943, 949
};

static int lobatto_tetra_face_indices_2_2[] = {
	36, 78, 84, 151, 157, 163, 244, 250, 256, 262, 370, 376, 382, 388, 394, 518, 524, 530, 536, 542, 548, 701, 707, 713, 719, 725, 731, 737, 908, 914, 920, 926, 932, 938, 944, 950
};

static int lobatto_tetra_face_indices_2_3[] = {
	37, 79, 85, 152, 158, 164, 245, 251, 257, 263, 371, 377, 383, 389, 395, 519, 525, 531, 537, 543, 549, 702, 708, 714, 720, 726, 732, 738, 909, 915, 921, 927, 933, 939, 945, 951
};

static int lobatto_tetra_face_indices_2_4[] = {
	38, 80, 86, 153, 159, 165, 246, 252, 258, 264, 372, 378, 384, 390, 396, 520, 526, 532, 538, 544, 550, 703, 709, 715, 721, 727, 733, 739, 910, 916, 922, 928, 934, 940, 946, 952
};

static int lobatto_tetra_face_indices_2_5[] = {
	39, 81, 87, 154, 160, 166, 247, 253, 259, 265, 373, 379, 385, 391, 397, 521, 527, 533, 539, 545, 551, 704, 710, 716, 722, 728, 734, 740, 911, 917, 923, 929, 935, 941, 947, 953
};

static int *lobatto_tetra_face_indices_2[] = {
	lobatto_tetra_face_indices_2_0,
	lobatto_tetra_face_indices_2_1,
	lobatto_tetra_face_indices_2_2,
	lobatto_tetra_face_indices_2_3,
	lobatto_tetra_face_indices_2_4,
	lobatto_tetra_face_indices_2_5
};

static int lobatto_tetra_face_indices_3_0[] = {
	40, 88, 94, 167, 173, 179, 266, 272, 278, 284, 398, 404, 410, 416, 422, 552, 558, 564, 570, 576, 582, 741, 747, 753, 759, 765, 771, 777, 954, 960, 966, 972, 978, 984, 990, 996
};

static int lobatto_tetra_face_indices_3_1[] = {
	41, 89, 95, 168, 174, 180, 267, 273, 279, 285, 399, 405, 411, 417, 423, 553, 559, 565, 571, 577, 583, 742, 748, 754, 760, 766, 772, 778, 955, 961, 967, 973, 979, 985, 991, 997
};

static int lobatto_tetra_face_indices_3_2[] = {
	42, 90, 96, 169, 175, 181, 268, 274, 280, 286, 400, 406, 412, 418, 424, 554, 560, 566, 572, 578, 584, 743, 749, 755, 761, 767, 773, 779, 956, 962, 968, 974, 980, 986, 992, 998
};

static int lobatto_tetra_face_indices_3_3[] = {
	43, 91, 97, 170, 176, 182, 269, 275, 281, 287, 401, 407, 413, 419, 425, 555, 561, 567, 573, 579, 585, 744, 750, 756, 762, 768, 774, 780, 957, 963, 969, 975, 981, 987, 993, 999
};

static int lobatto_tetra_face_indices_3_4[] = {
	44, 92, 98, 171, 177, 183, 270, 276, 282, 288, 402, 408, 414, 420, 426, 556, 562, 568, 574, 580, 586, 745, 751, 757, 763, 769, 775, 781, 958, 964, 970, 976, 982, 988, 994, 1000
};

static int lobatto_tetra_face_indices_3_5[] = {
	45, 93, 99, 172, 178, 184, 271, 277, 283, 289, 403, 409, 415, 421, 427, 557, 563, 569, 575, 581, 587, 746, 752, 758, 764, 770, 776, 782, 959, 965, 971, 977, 983, 989, 995, 1001
};

static int *lobatto_tetra_face_indices_3[] = {
	lobatto_tetra_face_indices_3_0,
	lobatto_tetra_face_indices_3_1,
	lobatto_tetra_face_indices_3_2,
	lobatto_tetra_face_indices_3_3,
	lobatto_tetra_face_indices_3_4,
	lobatto_tetra_face_indices_3_5
};

static int **lobatto_tetra_face_indices[] = {
	lobatto_tetra_face_indices_0,
	lobatto_tetra_face_indices_1,
	lobatto_tetra_face_indices_2,
	lobatto_tetra_face_indices_3,
};

static int lobatto_tetra_face_count[] = { 0, 0, 0, 1, 2, 3, 4, 5, 6, 7, 8 };

// bubbles //

static int lobatto_tetra_bubble_indices_all_orders[] = {
	100, 185, 186, 187, 290, 291, 292, 293, 294, 295, 428, 429, 430, 431, 432, 433, 434, 435, 436, 437, 588, 589, 590, 591, 592, 593, 594, 595, 596, 597, 598, 599, 600, 601, 602, 783, 784, 785, 786, 787, 788, 789, 790, 791, 792, 793, 794, 795, 796, 797, 798, 799, 800, 801, 802, 803, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012, 1013, 1014, 1015, 1016, 1017, 1018, 1019, 1020, 1021, 1022, 1023, 1024, 1025, 1026, 1027, 1028, 1029
};

static int *lobatto_tetra_bubble_indices[] = {
	NULL, NULL, NULL, NULL,
	lobatto_tetra_bubble_indices_all_orders,
	lobatto_tetra_bubble_indices_all_orders,
	lobatto_tetra_bubble_indices_all_orders,
	lobatto_tetra_bubble_indices_all_orders,
	lobatto_tetra_bubble_indices_all_orders,
	lobatto_tetra_bubble_indices_all_orders,
	lobatto_tetra_bubble_indices_all_orders
};

static int lobatto_tetra_bubble_count[] = { 0, 0, 0, 0, 1, 3, 6, 10, 15, 21, 28 };


static int lobatto_tetra_index_to_order[] = {
	1, 1, 1, 1,
	2, 2, 2, 2, 2, 2,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4,

	5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
	5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,
	5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5,

	6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
	6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,
	6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6,

	7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
	7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
	7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
	7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7, 7,

	8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
	8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
	8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
	8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
	8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,

	9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
	9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
	9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
	9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
	9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
	9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9,

	10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
	10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
	10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
	10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
	10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
	10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
	10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
	10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10,
	10, 10, 10, 10, 10, 10, 10, 10, 10, 10
};

extern shape_fn_t lobatto_tetra_dx[];
extern shape_fn_t lobatto_tetra_dy[];
extern shape_fn_t lobatto_tetra_dz[];

//

static shape_fn_t *lobatto_tetra_fn_table[] = {
	lobatto_tetra_fn
};

static shape_fn_t *lobatto_tetra_dx_table[] = {
	lobatto_tetra_dx
};

static shape_fn_t *lobatto_tetra_dy_table[] = {
	lobatto_tetra_dy
};

static shape_fn_t *lobatto_tetra_dz_table[] = {
	lobatto_tetra_dz
};
#endif


H1ShapesetLobattoTetra::H1ShapesetLobattoTetra() : Shapeset(2)
{
	_F_
#ifdef WITH_TETRA
	type = HERMES_H1_SPACE;
	mode = HERMES_MODE_TET;
	num_components = 1;

	// fn values are calculated by the tables
	shape_table[FN] = lobatto_tetra_fn_table;
	shape_table[DX]  = lobatto_tetra_dx_table;
	shape_table[DY]  = lobatto_tetra_dy_table;
	shape_table[DZ]  = lobatto_tetra_dz_table;
	shape_table[DXY] = NULL;
	shape_table[DXZ] = NULL;
	shape_table[DYZ] = NULL;

	vertex_indices = lobatto_tetra_vertex_indices;

	edge_indices = lobatto_tetra_edge_indices;
	edge_count = lobatto_tetra_edge_count;

	face_indices = lobatto_tetra_face_indices;
	face_count = lobatto_tetra_face_count;

	bubble_indices = lobatto_tetra_bubble_indices;
	bubble_count = lobatto_tetra_bubble_count;
#else
	EXIT(H3D_ERR_TETRA_NOT_COMPILED);
#endif
}


H1ShapesetLobattoTetra::~H1ShapesetLobattoTetra() {
	_F_
}

Ord3 H1ShapesetLobattoTetra::get_order(int index) const {
	_F_
#ifdef WITH_TETRA
	return Ord3(lobatto_tetra_index_to_order[index]);
#else
	return Ord3(0);
#endif
}

